from .choose_flags import (
    ChooseCursorFlag,
    ChooseItemFlag,
    ChooseListFlag,
    ChooseSelectedFlag,
)
from .confirm_flags import (
    ConfirmGeneralFlag,
    ConfirmPromptFlag,
    ConfirmSelectedFlag,
    ConfirmUnselectedFlag,
)
from .filter_flags import (
    FilterGeneralFlag,
    FilterIndicatorFlag,
    FilterMatchFlag,
    FilterPromptFlag,
    FilterTextFlag,
)
from .format_flags import FormatFlag
from .input_flags import InputCursorFlag, InputGeneralFlag, InputPromptFlag
from .join_flags import JoinTextFlag
from .spin_flags import SpinnerFlag, SpinnerTitleFlag, SpinSpinnerFlag
from .style_flags import StyleFlag
from .write_flags import (
    WriteBaseFlag,
    WriteCursorFlag,
    WriteCursorLineFlag,
    WriteCursorLineNumberFlag,
    WriteEndOfBuffer,
    WriteGeneralFlag,
    WriteLineNumberFlag,
    WritePlaceholderFlag,
    WritePromptFlag,
)



from abc import abstractmethod
from dataclasses import dataclass

from gumpython.constants import FlagSeparatorChoice, FlagTypeChoice
from gumpython.inputs import Input
from gumpython.utils import get_input_object


@dataclass
class Flag:
    name: str
    type: str
    _value: Input = None
    _display_name: str = None

    @property
    def flag(self):
        return f"--{self.name}"

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        input_obj = get_input_object(self.type, value, self.display_name)
        if input_obj.is_valid():
            self._value = input_obj

    @property
    def display_name(self):
        return self.name

    def get_command(self):
        return f"{self.flag}={self.value.compile()}"

    def __hash__(self):
        return hash((self.name,))


@dataclass
class SubFlag(Flag):
    sub_name: str = None
    separator: str = None

    @property
    def flag(self):
        return f"--{self.name}{self.separator}{self.sub_name}"

    @property
    def display_name(self):
        return f"{self.name} {self.sub_name}"

    def __hash__(self):
        return hash((self.name, self.sub_name))


class BaseFlag:
    def __init__(self):
        self.flag_name = self.get_flag_name()

        # align
        self.align = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.ALIGN,
            sub_name="align",
            separator=FlagSeparatorChoice.DOT,
        )
        self.margin = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.POSITION,
            sub_name="margin",
            separator=FlagSeparatorChoice.DOT,
        )
        self.padding = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.POSITION,
            sub_name="padding",
            separator=FlagSeparatorChoice.DOT,
        )

        # color
        self.background = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.COLOR,
            sub_name="background",
            separator=FlagSeparatorChoice.DOT,
        )
        self.foreground = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.COLOR,
            sub_name="foreground",
            separator=FlagSeparatorChoice.DOT,
        )

        # style
        self.bold = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.BOOL,
            sub_name="bold",
            separator=FlagSeparatorChoice.DOT,
        )
        self.faint = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.BOOL,
            sub_name="faint",
            separator=FlagSeparatorChoice.DOT,
        )
        self.italic = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.BOOL,
            sub_name="italic",
            separator=FlagSeparatorChoice.DOT,
        )
        self.strikethrough = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.BOOL,
            sub_name="strikethrough",
            separator=FlagSeparatorChoice.DOT,
        )
        self.underline = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.BOOL,
            sub_name="underline",
            separator=FlagSeparatorChoice.DOT,
        )

        # border
        self.border = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.BORDER,
            sub_name="border",
            separator=FlagSeparatorChoice.DOT,
        )
        self.border_background = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.COLOR,
            sub_name="border-background",
            separator=FlagSeparatorChoice.DOT,
        )
        self.border_foreground = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.COLOR,
            sub_name="border-foreground",
            separator=FlagSeparatorChoice.DOT,
        )

        # size
        self.height = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.INT,
            sub_name="height",
            separator=FlagSeparatorChoice.DOT,
        )
        self.width = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.INT,
            sub_name="width",
            separator=FlagSeparatorChoice.DOT,
        )

    @abstractmethod
    def get_flag_name(self):
        return ""



import re

from gumpython.exceptions import ColorInputError

from .inputs import Input


def is_valid_rgb(rgb: tuple):
    if not isinstance(rgb, tuple):
        raise ColorInputError(
            f"rgb should be of type 'tuple' but '{type(rgb).__name__}' given."
        )
    if len(rgb) != 3:
        raise ColorInputError(f"Expected (r, g, b) but given {rgb}")
    if rgb:
        for code in rgb:
            if code < 0 or code > 255:
                raise ColorInputError(f"{rgb} is not a valid rgb color code.")
        return True


def convert_rgb_to_hex(rgb: tuple):
    """
    It takes a tuple of three integers, representing the red, green, and blue values of a color, and returns a string
    containing the hexadecimal representation of that color

    :param rgb: tuple
    :type rgb: tuple
    :return: A string in the format of a hexadecimal color code.
    """
    return f"#{rgb[0]:02x}{rgb[1]:02x}{rgb[2]:02x}"


class Color(Input):
    def __init__(self, hex_code: str = None, flag_name: str = None):
        self.hex = hex_code
        self.flag_name = flag_name

    @classmethod
    def from_rgb(cls, rgb: tuple, flag_name):
        if is_valid_rgb(rgb):
            return cls(convert_rgb_to_hex(rgb), flag_name)

    def is_valid(self):
        return self._validate()

    def _validate(self):
        if not isinstance(self.hex, str):
            raise ColorInputError(
                f"Hex should be a string but '{type(self.hex).__name__}' given."
            )
        regex = "^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$"
        p = re.compile(regex)
        if re.search(p, self.hex):
            return True
        else:
            raise ColorInputError(f"{self.hex} is not a valid hex color code")

    def compile(self):
        if self.hex:
            return self.hex
        if self.rgb:
            return self._convert_rgb_to_hex()



from .alignment import Alignment
from .basic import IntegerInput, StringInput
from .border import Border
from .color import Color
from .format import Format
from .inputs import Input
from .position import Position
from .spinner import Spinner



import subprocess

from gumpython.arguments import SpinArguments
from gumpython.style import GumStyle

from .command import GumCommand


class Spin(GumCommand):
    def __init__(self, exec_command: str, show_output=False):
        super(Spin, self).__init__()
        self.argument = "spin"
        self.exec_command = exec_command
        self.show_output = show_output
        self.arguments = SpinArguments()

    def _compile_command(self):
        if self.show_output:
            self._add_to_flag_set(self.arguments.general.show_output, True)
        super(Spin, self)._compile_command()
        self.command.append(self.exec_command)

    def run(self):
        subprocess.run(self.get_command(), shell=True)

    def spinner(self, spinner_type: str):
        self._add_to_flag_set(self.arguments.spinner.spinner, spinner_type)
        return self

    def spinner_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.spinner)
        return self

    def title(self, text: str):
        self._add_to_flag_set(self.arguments.title.title, f"'{text}'")
        return self

    def title_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.title)
        return self



from abc import ABC, abstractmethod


class Input(ABC):
    @abstractmethod
    def is_valid(self):
        pass

    @abstractmethod
    def _validate(self):
        pass

    @abstractmethod
    def compile(self):
        pass



LINE = "line"
DOT = "dot"
MINI_DOT = "minidot"
JUMP = "jump"
PULSE = "pulse"
POINTS = "points"
GLOBE = "globe"
MOON = "moon"
MONKEY = "monkey"
METER = "meter"
HAMBURGER = "hamburger"


def get_all_spinner_types():
    return [
        LINE,
        DOT,
        MINI_DOT,
        JUMP,
        PULSE,
        POINTS,
        GLOBE,
        MOON,
        MONKEY,
        METER,
        HAMBURGER,
    ]



from gumpython.constants import FlagTypeChoice

from .flags import BaseFlag, Flag


class FilterIndicatorFlag(BaseFlag):
    def __init__(self):
        super(FilterIndicatorFlag, self).__init__()
        self.indicator = Flag(name=self.flag_name, type=FlagTypeChoice.STR)

    def get_flag_name(self):
        return "indicator"


class FilterMatchFlag(BaseFlag):
    def __init__(self):
        super(FilterMatchFlag, self).__init__()

    def get_flag_name(self):
        return "match"


class FilterPromptFlag(BaseFlag):
    def __init__(self):
        super(FilterPromptFlag, self).__init__()
        self.prompt = Flag(name=self.flag_name, type=FlagTypeChoice.STR)

    def get_flag_name(self):
        return "prompt"


class FilterTextFlag(BaseFlag):
    def __init__(self):
        super(FilterTextFlag, self).__init__()

    def get_flag_name(self):
        return "text"


class FilterGeneralFlag:
    def __init__(self):
        self.height = Flag(name="height", type=FlagTypeChoice.INT)
        self.placeholder = Flag(name="placeholder", type=FlagTypeChoice.STR)
        self.width = Flag(name="width", type=FlagTypeChoice.INT)



from gumpython.exceptions import PositionInputError

from .inputs import Input


class Position(Input):
    def __init__(self, value: tuple, flag_name: str):
        self.value = value
        self.flag_name = flag_name

    def is_valid(self):
        return self._validate()

    def _validate(self):
        if not isinstance(self.value, tuple):
            raise PositionInputError(
                f"'tuple' is expected for {self.flag_name} but '{type(self.value).__name__}' is given."
            )
        if len(self.value) != 2:
            raise PositionInputError(
                f"'(x, y)' expected for {self.flag_name} but '{self.value}' is given."
            )
        if not isinstance(self.value[0], int) or not isinstance(
            self.value[1], int
        ):
            raise PositionInputError(
                f"'int' values expected for {self.flag_name}."
            )
        return True

    def compile(self):
        return f"{self.value[0]} {self.value[1]}"



from typing import List

from gumpython.arguments import FilterArguments
from gumpython.exceptions import FilterArgumentError
from gumpython.style import GumStyle

from .command import GumCommand


class Filter(GumCommand):
    def __init__(self, item_list: List[str]):
        super(Filter, self).__init__()
        self.argument = "filter"
        self.item_list = item_list
        self._validate_item_list()
        self.input = "\n".join(self.item_list)
        self.arguments = FilterArguments()

    def _validate_item_list(self):
        if not isinstance(self.item_list, list):
            raise FilterArgumentError(
                f"'list' expected for item list argument but '{type(self.item_list).__name__}' is given."
            )
        if not self.item_list:
            raise FilterArgumentError("'item_list' can not be an empty list.")
        for item in self.item_list:
            if not isinstance(item, str):
                raise FilterArgumentError(f"{item} is not a string value.")
        return True

    def indicator(self, indicator: str):
        self._add_to_flag_set(self.arguments.indicator.indicator, indicator)
        return self

    def indicator_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.indicator)
        return self

    def match_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.match)
        return self

    def prompt(self, prompt: str):
        self._add_to_flag_set(self.arguments.prompt.prompt, prompt)
        return self

    def prompt_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.prompt)
        return self

    def text_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.text)
        return self

    def placeholder(self, placeholder: str):
        self._add_to_flag_set(self.arguments.general.placeholder, placeholder)
        return self

    def input_size(self, width: int = None, height: int = None):
        self._size(self.arguments.general, width, height)
        return self



from gumpython.constants import FlagSeparatorChoice, FlagTypeChoice

from .flags import Flag, SubFlag


class StyleFlag:
    def __init__(self):
        self.border = Flag(name="border", type=FlagTypeChoice.BORDER)
        self.border_background = SubFlag(
            name="border",
            type=FlagTypeChoice.COLOR,
            sub_name="background",
            separator=FlagSeparatorChoice.HYPHEN,
        )
        self.border_foreground = SubFlag(
            name="border",
            type=FlagTypeChoice.COLOR,
            sub_name="foreground",
            separator=FlagSeparatorChoice.HYPHEN,
        )
        self.align = Flag(name="align", type=FlagTypeChoice.ALIGN)
        self.background = Flag(name="background", type=FlagTypeChoice.COLOR)
        self.foreground = Flag(name="foreground", type=FlagTypeChoice.COLOR)
        self.bold = Flag(name="bold", type=FlagTypeChoice.BOOL)
        self.faint = Flag(name="faint", type=FlagTypeChoice.BOOL)
        self.height = Flag(name="height", type=FlagTypeChoice.INT)
        self.italic = Flag(name="italic", type=FlagTypeChoice.BOOL)
        self.strikethrough = Flag(
            name="strikethrough", type=FlagTypeChoice.BOOL
        )
        self.underline = Flag(name="underline", type=FlagTypeChoice.BOOL)
        self.margin = Flag(name="margin", type=FlagTypeChoice.POSITION)
        self.padding = Flag(name="padding", type=FlagTypeChoice.POSITION)
        self.width = Flag(name="width", type=FlagTypeChoice.INT)



class FlagTypeChoice:
    COLOR = "color"
    INT = "integer"
    STR = "string"
    BOOL = "boolean"
    ALIGN = "alignment"
    POSITION = "position"
    BORDER = "border"
    FORMAT = "format"
    SPINNER = "spinner"


class FlagSeparatorChoice:
    DOT = "."
    HYPHEN = "-"



from typing import List

from gumpython.arguments import ChooseArgument
from gumpython.exceptions import ChooseArgumentError
from gumpython.style import GumStyle

from .command import GumCommand


class Choose(GumCommand):
    def __init__(self, item_list: List[str]):
        super().__init__()
        self.argument = "choose"
        self.returning = True
        self.item_list = item_list
        self._validate_item_list()
        self.arguments = ChooseArgument()

    def _validate_item_list(self):
        if not isinstance(self.item_list, list):
            raise ChooseArgumentError(
                f"'list' expected for item list argument but '{type(self.item_list).__name__}' is given."
            )
        for item in self.item_list:
            if not isinstance(item, str):
                raise ChooseArgumentError(f"{item} is not a string value.")
        return True

    def _compile_command(self):
        super(Choose, self)._compile_command()
        self.command.extend(self.item_list)

    def run(self):
        response = super(Choose, self).run()
        return response.split("\n")[:-1]

    def cursor(self, cursor_text: str):
        self._add_to_flag_set(self.arguments.cursor.cursor, cursor_text)
        return self

    def cursor_prefix(self, prefix: str):
        self._add_to_flag_set(self.arguments.cursor.prefix, prefix)
        return self

    def cursor_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.cursor)
        return self

    def items_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.item)
        return self

    def selected_prefix(self, prefix: str):
        self._add_to_flag_set(self.arguments.selected.prefix, prefix)
        return self

    def selected_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.selected)
        return self

    def list_height(self, height: int):
        self._add_to_flag_set(self.arguments.list.height, height)
        return self

    def limit(self, limit: int):
        self._add_to_flag_set(self.arguments.list.limit, limit)
        return self

    def no_limit(self):
        self._add_to_flag_set(self.arguments.list.no_limit, True)
        return self

    def unselected_prefix(self, prefix: str):
        self._add_to_flag_set(self.arguments.list.unselected_prefix, prefix)
        return self



from gumpython.exceptions import AlignmentInputError

from .help_inputs.alignment_types import all_alignment_types
from .inputs import Input


class Alignment(Input):
    def __init__(self, alignment_type: str, flag_name: str):
        self.alignment_type = alignment_type
        self.flag_name = flag_name

    def is_valid(self):
        return self._validate()

    def _validate(self):
        if not isinstance(self.alignment_type, str):
            raise AlignmentInputError(
                f"'str' expected for {self.flag_name} but '{type(self.alignment_type).__name__}' is given."
            )
        if self.alignment_type not in all_alignment_types():
            raise AlignmentInputError(
                f"Alignment must be one of {all_alignment_types()} but '{self.alignment_type}' is given."
            )
        return True

    def compile(self):
        return self.alignment_type



from gumpython.arguments import FormatArguments
from gumpython.exceptions import FormatArgumentError

from .command import GumCommand


class Format(GumCommand):
    def __init__(self, text):
        super(Format, self).__init__()
        self.argument = "format"
        self.arguments = FormatArguments()
        self.text = text
        self._validate_text()

    def _validate_text(self):
        if not isinstance(self.text, str):
            raise FormatArgumentError(
                f"'text' should be of type 'str' but '{type(self.text).__name__}' given."
            )

    def _compile_command(self):
        super(Format, self)._compile_command()
        self.command.append(self.text)

    def type(self, format_type: str):
        self._add_to_flag_set(self.arguments.type.type, format_type)
        return self



from .choose import Choose
from .confirm import Confirm
from .filter import Filter
from .format import Format
from .input import Input
from .join import Join
from .spin import Spin
from .style import Style
from .write import Write



from gumpython.exceptions import ColorInputError
from gumpython.inputs import (
    Alignment,
    Border,
    Color,
    Format,
    IntegerInput,
    Position,
    Spinner,
    StringInput,
)

from .constants import FlagTypeChoice


def get_color_object(value, flag_name):
    if isinstance(value, str):
        return Color(hex_code=value, flag_name=flag_name)
    if isinstance(value, tuple):
        return Color.from_rgb(value, flag_name=flag_name)
    raise ColorInputError(
        f"Expected 'str' or 'tuple' input for color but '{type(value).__name__}' given."
    )


def get_border_object(border_type, flag_name):
    return Border(border_type, flag_name)


def get_string_input_object(value, flag_name):
    return StringInput(value, flag_name)


def get_integer_input_object(value, flag_name):
    return IntegerInput(value, flag_name)


def get_alignment_object(value, flag_name):
    return Alignment(value, flag_name)


def get_position_object(value, flag_name):
    return Position(value, flag_name)


def get_format_object(value, flag_name):
    return Format(value, flag_name)


def get_spinner_object(value, flag_name):
    return Spinner(value, flag_name)


FLAG_TYPE_INPUT_MAPPING = {
    FlagTypeChoice.COLOR: get_color_object,
    FlagTypeChoice.INT: get_integer_input_object,
    FlagTypeChoice.STR: get_string_input_object,
    FlagTypeChoice.ALIGN: get_alignment_object,
    FlagTypeChoice.POSITION: get_position_object,
    FlagTypeChoice.BORDER: get_border_object,
    FlagTypeChoice.FORMAT: get_format_object,
    FlagTypeChoice.SPINNER: get_spinner_object,
}


def get_input_object(flag_type, value, flag_name):
    func = FLAG_TYPE_INPUT_MAPPING.get(flag_type)
    return func(value, flag_name)



from gumpython.constants import FlagTypeChoice

from .flags import BaseFlag, Flag


class WriteBaseFlag(BaseFlag):
    def __init__(self):
        super(WriteBaseFlag, self).__init__()

    def get_flag_name(self):
        return "base"


class WriteCursorLineNumberFlag(BaseFlag):
    def __init__(self):
        super(WriteCursorLineNumberFlag, self).__init__()

    def get_flag_name(self):
        return "cursor-line-number"


class WriteCursorLineFlag(BaseFlag):
    def __init__(self):
        super(WriteCursorLineFlag, self).__init__()

    def get_flag_name(self):
        return "cursor-line"


class WriteCursorFlag(BaseFlag):
    def __init__(self):
        super(WriteCursorFlag, self).__init__()

    def get_flag_name(self):
        return "cursor"


class WriteEndOfBuffer(BaseFlag):
    def __init__(self):
        super(WriteEndOfBuffer, self).__init__()

    def get_flag_name(self):
        return "end-of-buffer"


class WriteLineNumberFlag(BaseFlag):
    def __init__(self):
        super(WriteLineNumberFlag, self).__init__()

    def get_flag_name(self):
        return "line-number"


class WritePlaceholderFlag(BaseFlag):
    def __init__(self):
        super(WritePlaceholderFlag, self).__init__()
        self.placeholder = Flag(name=self.flag_name, type=FlagTypeChoice.STR)

    def get_flag_name(self):
        return "placeholder"


class WritePromptFlag(BaseFlag):
    def __init__(self):
        super(WritePromptFlag, self).__init__()
        self.prompt = Flag(name=self.flag_name, type=FlagTypeChoice.STR)

    def get_flag_name(self):
        return "prompt"


class WriteGeneralFlag:
    def __init__(self):
        self.char_limit = Flag(name="char-limit", type=FlagTypeChoice.INT)
        self.height = Flag(name="height", type=FlagTypeChoice.INT)
        self.show_cursor_line = Flag(
            name="show-cursor-line", type=FlagTypeChoice.BOOL
        )
        self.show_line_numbers = Flag(
            name="show-line-numbers", type=FlagTypeChoice.BOOL
        )
        self.value = Flag(name="value", type=FlagTypeChoice.STR)
        self.width = Flag(name="width", type=FlagTypeChoice.INT)



from gumpython.exceptions import SpinnerInputError

from .help_inputs.spinner_types import get_all_spinner_types
from .inputs import Input


class Spinner(Input):
    def __init__(self, spinner_type, flag_name):
        self.spinner_type = spinner_type
        self.flag_name = flag_name

    def is_valid(self):
        return self._validate()

    def _validate(self):
        if not isinstance(self.spinner_type, str):
            raise SpinnerInputError(
                f"'str' input expected for {self.flag_name} format type but '{type(self.spinner_type).__name__}' is given"
            )
        if self.spinner_type not in get_all_spinner_types():
            raise SpinnerInputError(
                f"border must be one of {get_all_spinner_types()} but got '{self.spinner_type}'"
            )
        return True

    def compile(self):
        return self.spinner_type



from dataclasses import asdict, dataclass, field, fields
from typing import Union


@dataclass
class Border:
    border: str = None
    border_foreground: Union[str, tuple] = None
    border_background: Union[str, tuple] = None


@dataclass
class Alignment:
    align: str = None
    margin: tuple = None
    padding: tuple = None


@dataclass
class Color:
    foreground: Union[str, tuple] = None
    background: Union[str, tuple] = None


@dataclass
class FontStyle:
    bold: bool = False
    italic: bool = False
    faint: bool = False
    underline: bool = False
    strikethrough: bool = False


@dataclass
class Size:
    width: int = None
    height: int = None


@dataclass
class GumStyle:
    border: Border = field(default_factory=Border)
    alignment: Alignment = field(default_factory=Alignment)
    color: Color = field(default_factory=Color)
    font_style: FontStyle = field(default_factory=FontStyle)
    size: Size = field(default_factory=Size)


def get_styles(style: GumStyle):
    styles = {}
    for f in fields(style):
        attr = getattr(style, f.name)
        styles.update(
            asdict(
                attr,
                dict_factory=lambda x: {k: v for (k, v) in x if v is not None},
            )
        )
    return styles



from gumpython.constants import FlagTypeChoice
from gumpython.exceptions import InputError
from gumpython.run_command import run_command
from gumpython.style import GumStyle, get_styles


class GumCommand:
    def __init__(self):
        self.command = None
        self.argument = None
        self.__flag_set = set()
        self.returning = False
        self.input = None

    def run(self):
        self._compile_command()
        response = run_command(self.command, self.input)
        if self.returning:
            return response.stdout
        print(response.stdout)

    def get_command(self):
        self._compile_command()
        return " ".join(self.command)

    def _add_to_flag_set(self, flag, value):
        if flag.type != FlagTypeChoice.BOOL:
            flag.value = value
            self.__flag_set.add(flag)
        else:
            if not isinstance(value, bool):
                raise InputError(
                    f"'bool' expected for {flag.display_name} but '{type(value).__name__}' given."
                )
            if value:
                self.__flag_set.add(flag)

    def _compile_style(self, style: GumStyle, flag):
        styles = get_styles(style)
        for style, value in styles.items():
            self._add_to_flag_set(getattr(flag, style), value)

    def _compile_command(self):
        self.command = ["gum", self.argument]
        for flag in self.__flag_set:
            if flag.type == FlagTypeChoice.BOOL:
                self.command.append(flag.flag)
            else:
                self.command.append(flag.get_command())

    def __str__(self):
        return self.get_command()



from gumpython.commands import (
    Choose,
    Confirm,
    Filter,
    Format,
    Input,
    Join,
    Spin,
    Style,
    Write,
)
from gumpython.inputs.help_inputs import alignment_types as alignment
from gumpython.inputs.help_inputs import border_types as border
from gumpython.inputs.help_inputs import color_names as color
from gumpython.inputs.help_inputs import format_types as text_format
from gumpython.inputs.help_inputs import spinner_types as spinner
from gumpython.style import GumStyle

from .__version__ import version as __version__



from gumpython.constants import FlagTypeChoice

from .flags import Flag


class JoinTextFlag:
    def __init__(self):
        self.align = Flag(name="align", type=FlagTypeChoice.ALIGN)
        self.horizontal = Flag(name="horizontal", type=FlagTypeChoice.BOOL)
        self.vertical = Flag(name="vertical", type=FlagTypeChoice.BOOL)



LEFT = "left"
CENTER = "center"
RIGHT = "right"
BOTTOM = "bottom"
MIDDLE = "middle"
TOP = "top"


def all_alignment_types():
    return [
        LEFT,
        CENTER,
        RIGHT,
        BOTTOM,
        MIDDLE,
        TOP,
    ]



from gumpython.flags import (
    ChooseCursorFlag,
    ChooseItemFlag,
    ChooseListFlag,
    ChooseSelectedFlag,
    ConfirmGeneralFlag,
    ConfirmPromptFlag,
    ConfirmSelectedFlag,
    ConfirmUnselectedFlag,
    FilterGeneralFlag,
    FilterIndicatorFlag,
    FilterMatchFlag,
    FilterPromptFlag,
    FilterTextFlag,
    FormatFlag,
    InputCursorFlag,
    InputGeneralFlag,
    InputPromptFlag,
    JoinTextFlag,
    SpinnerFlag,
    SpinnerTitleFlag,
    SpinSpinnerFlag,
    StyleFlag,
    WriteBaseFlag,
    WriteCursorFlag,
    WriteCursorLineFlag,
    WriteCursorLineNumberFlag,
    WriteEndOfBuffer,
    WriteGeneralFlag,
    WriteLineNumberFlag,
    WritePlaceholderFlag,
    WritePromptFlag,
)


class Argument:
    pass


class StyleArgument(Argument):
    def __init__(self):
        self.args = StyleFlag()


class ChooseArgument(Argument):
    def __init__(self):
        self.cursor = ChooseCursorFlag()
        self.item = ChooseItemFlag()
        self.selected = ChooseSelectedFlag()
        self.list = ChooseListFlag()


class ConfirmArguments(Argument):
    def __init__(self):
        self.general = ConfirmGeneralFlag()
        self.prompt = ConfirmPromptFlag()
        self.selected = ConfirmSelectedFlag()
        self.unselected = ConfirmUnselectedFlag()


class FilterArguments(Argument):
    def __init__(self):
        self.general = FilterGeneralFlag()
        self.match = FilterMatchFlag()
        self.prompt = FilterPromptFlag()
        self.indicator = FilterIndicatorFlag()
        self.text = FilterTextFlag()


class FormatArguments(Argument):
    def __init__(self):
        self.type = FormatFlag()


class InputArguments(Argument):
    def __init__(self):
        self.general = InputGeneralFlag()
        self.cursor = InputCursorFlag()
        self.prompt = InputPromptFlag()


class JoinArguments(Argument):
    def __init__(self):
        self.text = JoinTextFlag()


class SpinArguments(Argument):
    def __init__(self):
        self.general = SpinnerFlag()
        self.spinner = SpinSpinnerFlag()
        self.title = SpinnerTitleFlag()


class WriteArguments(Argument):
    def __init__(self):
        self.base = WriteBaseFlag()
        self.cursor = WriteCursorFlag()
        self.cursor_line = WriteCursorLineFlag()
        self.cursor_line_number = WriteCursorLineNumberFlag()
        self.end_of_buffer = WriteEndOfBuffer()
        self.general = WriteGeneralFlag()
        self.line_number = WriteLineNumberFlag()
        self.placeholder = WritePlaceholderFlag()
        self.prompt = WritePromptFlag()



from gumpython.arguments import WriteArguments
from gumpython.style import GumStyle

from .command import GumCommand


class Write(GumCommand):
    def __init__(self):
        super(Write, self).__init__()
        self.argument = "write"
        self.returning = True
        self.arguments = WriteArguments()

    def run(self):
        response = super(Write, self).run()
        return list(filter(lambda x: x != "", response.split("\n")))

    def base_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.base)
        return self

    def cursor_line_number_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.cursor_line_number)
        return self

    def cursor_line_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.cursor_line)
        return self

    def cursor_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.cursor)
        return self

    def end_of_buffer_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.end_of_buffer)
        return self

    def line_number_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.line_number)
        return self

    def placeholder(self, placeholder: str):
        self._add_to_flag_set(
            self.arguments.placeholder.placeholder, placeholder
        )
        return self

    def placeholder_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.placeholder)
        return self

    def prompt(self, prompt: str):
        self._add_to_flag_set(self.arguments.prompt.prompt, prompt)
        return self

    def prompt_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.prompt)
        return self

    def char_limit(self, char_limit: int):
        self._add_to_flag_set(self.arguments.general.char_limit, char_limit)
        return self

    def height(self, height: int):
        self._add_to_flag_set(self.arguments.general.height, height)
        return self

    def show_cursor_line(self):
        self._add_to_flag_set(self.arguments.general.show_cursor_line, True)
        return self

    def show_line_numbers(self):
        self._add_to_flag_set(self.arguments.general.show_line_numbers, True)
        return self

    def value(self, value: str):
        self._add_to_flag_set(self.arguments.general.value, value)
        return self

    def width(self, width: int):
        self._add_to_flag_set(self.arguments.general.width, width)
        return self



from gumpython.arguments import ConfirmArguments
from gumpython.exceptions import ConfirmArgumentError
from gumpython.run_command import run_command
from gumpython.style import GumStyle

from .command import GumCommand


class Confirm(GumCommand):
    def __init__(self, message: str):
        super(Confirm, self).__init__()
        self.argument = "confirm"
        self.message = message
        self._validate_message()
        self.arguments = ConfirmArguments()
        self.returning = True

    def _compile_command(self):
        super(Confirm, self)._compile_command()
        self.command.append(self.message)

    def _validate_message(self):
        if not isinstance(self.message, str):
            raise ConfirmArgumentError(
                f"'str' expected for message argument but '{type(self.message).__name__}' is given."
            )

    def run(self):
        self._compile_command()
        response = run_command(self.command, self.input)
        return response.returncode == 0

    def affirmative(self, text: str):
        self._add_to_flag_set(self.arguments.general.affirmative, text)
        return self

    def negative(self, text: str):
        self._add_to_flag_set(self.arguments.general.negative, text)
        return self

    def prompt_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.prompt)
        return self

    def selected_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.selected)
        return self

    def unselected_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.unselected)
        return self



from gumpython.constants import FlagTypeChoice

from .flags import BaseFlag, Flag


class SpinSpinnerFlag(BaseFlag):
    def __init__(self):
        super(SpinSpinnerFlag, self).__init__()
        self.spinner = Flag(name=self.flag_name, type=FlagTypeChoice.SPINNER)

    def get_flag_name(self):
        return "spinner"


class SpinnerTitleFlag(BaseFlag):
    def __init__(self):
        super(SpinnerTitleFlag, self).__init__()
        self.title = Flag(name=self.flag_name, type=FlagTypeChoice.STR)

    def get_flag_name(self):
        return "title"


class SpinnerFlag:
    def __init__(self):
        self.show_output = Flag(name="show-output", type=FlagTypeChoice.BOOL)



from gumpython.arguments import InputArguments
from gumpython.style import GumStyle

from .command import GumCommand


class Input(GumCommand):
    def __init__(self):
        super(Input, self).__init__()
        self.argument = "input"
        self.arguments = InputArguments()
        self.returning = True

    def run(self):
        response = super(Input, self).run()
        return response[:-1]

    def char_limit(self, limit: int):
        self._add_to_flag_set(self.arguments.general.char_limit, limit)
        return self

    def password(self):
        self._add_to_flag_set(self.arguments.general.password, True)
        return self

    def placeholder(self, placeholder: str):
        self._add_to_flag_set(self.arguments.general.placeholder, placeholder)
        return self

    def value(self, value: str):
        self._add_to_flag_set(self.arguments.general.value, value)
        return self

    def width(self, width: int):
        self._add_to_flag_set(self.arguments.general.width, width)
        return self

    def cursor_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.cursor)
        return self

    def prompt(self, prompt: str, style: GumStyle = None):
        self._add_to_flag_set(self.arguments.prompt.prompt, prompt)
        return self

    def prompt_style(self, style: GumStyle):
        self._compile_style(style, self.arguments.prompt)



from gumpython.exceptions import InputError

from .inputs import Input


class IntegerInput(Input):
    def __init__(self, value: int, flag_name):
        self.value = value
        self.flag_name = flag_name

    def is_valid(self):
        return self._validate()

    def _validate(self):
        if isinstance(self.value, int):
            return True
        raise InputError(
            f"'int' input expected for {self.flag_name} but given '{type(self.value).__name__}'"
        )

    def compile(self):
        return self.value


class StringInput(Input):
    def __init__(self, value: str, flag_name):
        self.value = value
        self.flag_name = flag_name

    def is_valid(self):
        return self._validate()

    def _validate(self):
        if isinstance(self.value, str):
            return True
        raise InputError(
            f"'str' input expected for {self.flag_name} but given '{type(self.value).__name__}'"
        )

    def compile(self):
        return self.value



ALICEBLUE = "#f0f8ff"
ANTIQUEWHITE = "#faebd7"
AQUA = "#00ffff"
AQUAMARINE = "#7fffd4"
AZURE = "#f0ffff"
BEIGE = "#f5f5dc"
BISQUE = "#ffe4c4"
BLACK = "#000000"
BLANCHEDALMOND = "#ffebcd"
BLUE = "#0000ff"
BLUEVIOLET = "#8a2be2"
BROWN = "#a52a2a"
BURLYWOOD = "#deb887"
CADETBLUE = "#5f9ea0"
CHARTREUSE = "#7fff00"
CHOCOLATE = "#d2691e"
CORAL = "#ff7f50"
CORNFLOWERBLUE = "#6495ed"
CORNSILK = "#fff8dc"
CRIMSON = "#dc143c"
CYAN = "#00ffff"
DARKBLUE = "#00008b"
DARKCYAN = "#008b8b"
DARKGOLDENROD = "#b8860b"
DARKGRAY = "#a9a9a9"
DARKGREEN = "#006400"
DARKGREY = "#a9a9a9"
DARKKHAKI = "#bdb76b"
DARKMAGENTA = "#8b008b"
DARKOLIVEGREEN = "#556b2f"
DARKORANGE = "#ff8c00"
DARKORCHID = "#9932cc"
DARKRED = "#8b0000"
DARKSALMON = "#e9967a"
DARKSEAGREEN = "#8fbc8f"
DARKSLATEBLUE = "#483d8b"
DARKSLATEGRAY = "#2f4f4f"
DARKSLATEGREY = "#2f4f4f"
DARKTURQUOISE = "#00ced1"
DARKVIOLET = "#9400d3"
DEEPPINK = "#ff1493"
DEEPSKYBLUE = "#00bfff"
DIMGRAY = "#696969"
DIMGREY = "#696969"
ODGERBLUE = "#1e90ff"
FIREBRICK = "#b22222"
FLORALWHITE = "#fffaf0"
FORESTGREEN = "#228b22"
FUCHSIA = "#ff00ff"
GAINSBORO = "#dcdcdc"
GHOSTWHITE = "#f8f8ff"
GOLD = "#ffd700"
GOLDENROD = "#daa520"
GRAY = "#808080"
GREEN = "#008000"
GREENYELLOW = "#adff2f"
GREY = "#808080"
HONEYDEW = "#f0fff0"
HOTPINK = "#ff69b4"
INDIANRED = "#cd5c5c"
INDIGO = "#4b0082"
IVORY = "#fffff0"
KHAKI = "#f0e68c"
LAVENDER = "#e6e6fa"
LAVENDERBLUSH = "#fff0f5"
LAWNGREEN = "#7cfc00"
LEMONCHIFFON = "#fffacd"
LIGHTBLUE = "#add8e6"
LIGHTCORAL = "#f08080"
LIGHTCYAN = "#e0ffff"
LIGHTGOLDENRODYELLOW = "#fafad2"
LIGHTGRAY = "#d3d3d3"
LIGHTGREEN = "#90ee90"
LIGHTGREY = "#d3d3d3"
LIGHTPINK = "#ffb6c1"
LIGHTSALMON = "#ffa07a"
LIGHTSEAGREEN = "#20b2aa"
LIGHTSKYBLUE = "#87cefa"
LIGHTSLATEGRAY = "#778899"
LIGHTSLATEGREY = "#778899"
LIGHTSTEELBLUE = "#b0c4de"
LIGHTYELLOW = "#ffffe0"
LIME = "#00ff00"
LIMEGREEN = "#32cd32"
LINEN = "#faf0e6"
MAGENTA = "#ff00ff"
MAROON = "#800000"
MEDIUMAQUAMARINE = "#66cdaa"
MEDIUMBLUE = "#0000cd"
MEDIUMORCHID = "#ba55d3"
MEDIUMPURPLE = "#9370db"
MEDIUMSEAGREEN = "#3cb371"
MEDIUMSLATEBLUE = "#7b68ee"
MEDIUMSPRINGGREEN = "#00fa9a"
MEDIUMTURQUOISE = "#48d1cc"
MEDIUMVIOLETRED = "#c71585"
MIDNIGHTBLUE = "#191970"
MINTCREAM = "#f5fffa"
MISTYROSE = "#ffe4e1"
MOCCASIN = "#ffe4b5"
NAVAJOWHITE = "#ffdead"
NAVY = "#000080"
OLDLACE = "#fdf5e6"
OLIVE = "#808000"
OLIVEDRAB = "#6b8e23"
ORANGE = "#ffa500"
ORANGERED = "#ff4500"
ORCHID = "#da70d6"
PALEGOLDENROD = "#eee8aa"
PALEGREEN = "#98fb98"
PALETURQUOISE = "#afeeee"
PALEVIOLETRED = "#db7093"
PAPAYAWHIP = "#ffefd5"
PEACHPUFF = "#ffdab9"
PERU = "#cd853f"
PINK = "#ffc0cb"
PLUM = "#dda0dd"
POWDERBLUE = "#b0e0e6"
PURPLE = "#800080"
RED = "#ff0000"
ROSYBROWN = "#bc8f8f"
ROYALBLUE = "#4169e1"
SADDLEBROWN = "#8b4513"
SALMON = "#fa8072"
SANDYBROWN = "#f4a460"
SEAGREEN = "#2e8b57"
SEASHELL = "#fff5ee"
SIENNA = "#a0522d"
SILVER = "#c0c0c0"
SKYBLUE = "#87ceeb"
SLATEBLUE = "#6a5acd"
SLATEGRAY = "#708090"
SLATEGREY = "#708090"
SNOW = "#fffafa"
SPRINGGREEN = "#00ff7f"
STEELBLUE = "#4682b4"
TAN = "#d2b48c"
TEAL = "#008080"
THISTLE = "#d8bfd8"
TOMATO = "#ff6347"
TURQUOISE = "#40e0d0"
VIOLET = "#ee82ee"
WHEAT = "#f5deb3"
WHITE = "#ffffff"
WHITESMOKE = "#f5f5f5"
YELLOW = "#ffff00"
YELLOWGREEN = "#9acd32"



from gumpython.constants import FlagSeparatorChoice, FlagTypeChoice

from .flags import BaseFlag, Flag, SubFlag


class ChooseCursorFlag(BaseFlag):
    def __init__(self):
        super().__init__()
        # main
        self.cursor = Flag(name=self.flag_name, type=FlagTypeChoice.STR)
        self.prefix = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.STR,
            sub_name="prefix",
            separator=FlagSeparatorChoice.HYPHEN,
        )

    def get_flag_name(self):
        return "cursor"


class ChooseItemFlag(BaseFlag):
    def __init__(self):
        super(ChooseItemFlag, self).__init__()

    def get_flag_name(self):
        return "item"


class ChooseSelectedFlag(BaseFlag):
    def __init__(self):
        super(ChooseSelectedFlag, self).__init__()
        self.prefix = SubFlag(
            name=self.flag_name,
            type=FlagTypeChoice.STR,
            sub_name="prefix",
            separator=FlagSeparatorChoice.HYPHEN,
        )

    def get_flag_name(self):
        return "selected"


class ChooseListFlag:
    def __init__(self):
        self.height = Flag(name="height", type=FlagTypeChoice.INT)
        self.limit = Flag(name="limit", type=FlagTypeChoice.INT)
        self.no_limit = Flag(name="no-limit", type=FlagTypeChoice.BOOL)
        self.unselected_prefix = Flag(
            name="unselected-prefix", type=FlagTypeChoice.STR
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gumpython',
 'gumpython.arguments',
 'gumpython.commands',
 'gumpython.flags',
 'gumpython.inputs',
 'gumpython.inputs.help_inputs']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'gumpython',
    'version': '0.3.0',
    'description': 'A python library for gum cli',
    'long_description': '# GumPython\n A python library for gum cli\n\n## Requirements\n\n- [gum cli](https://github.com/charmbracelet/gum)\n- Python >= 3.8\n\n## How to Use\n\n```python\nfrom gumpython import Style, color, alignment, border\n\nstyle = Style(["Hi there!", "My name is GumPython :)"]) \\\n            .border(border.ROUNDED, foreground_color=color.FUCHSIA) \\\n            .text_color(foreground=color.AQUA) \\\n            .align(alignment=alignment.CENTER) \\\n            .text_font(bold=True, italic=True)\n\nstyle.run()\n```',
    'author': 'Wasi Haider',
    'author_email': 'wsi.haidr@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from gumpython.constants import FlagTypeChoice

from .flags import BaseFlag, Flag


class ConfirmPromptFlag(BaseFlag):
    def __init__(self):
        super(ConfirmPromptFlag, self).__init__()

    def get_flag_name(self):
        return "prompt"


class ConfirmSelectedFlag(BaseFlag):
    def __init__(self):
        super(ConfirmSelectedFlag, self).__init__()

    def get_flag_name(self):
        return "selected"


class ConfirmUnselectedFlag(BaseFlag):
    def __init__(self):
        super(ConfirmUnselectedFlag, self).__init__()

    def get_flag_name(self):
        return "unselected"


class ConfirmGeneralFlag:
    def __init__(self):
        self.affirmative = Flag(name="affirmative", type=FlagTypeChoice.STR)
        self.negative = Flag(name="negative", type=FlagTypeChoice.STR)



import subprocess
from typing import List, Union

from gumpython.arguments import JoinArguments
from gumpython.exceptions import JoinArgumentError

from .command import GumCommand
from .style import Style


class Join(GumCommand):
    def __init__(self, items=List[Union[str, GumCommand]]):
        super(Join, self).__init__()
        self.argument = "join"
        self.items = items
        self.__parsed_items = []
        self._validate_and_parse_items()
        self.arguments = JoinArguments()
        self.__vertical = False

    def _validate_and_parse_items(self):
        if not self.items:
            raise JoinArgumentError("Items can not be empty.")
        if not isinstance(self.items, list):
            raise JoinArgumentError(
                f"'list' expected for items but '{type(self.items).__name__}' given."
            )
        for item in self.items:
            if isinstance(item, str):
                self.__parsed_items.append(item)
            elif isinstance(item, (Style, self.__class__)):
                self.__parsed_items.append(f'"$({item.get_command()})"')
            else:
                raise JoinArgumentError(
                    f"{item} is not of type 'str' or 'gumpython.Style'"
                )

    def _compile_command(self):
        self._add_to_flag_set(
            self.arguments.text.vertical
            if self.__vertical
            else self.arguments.text.horizontal,
            True,
        )
        super(Join, self)._compile_command()
        self.command.extend(self.__parsed_items)

    def run(self):
        subprocess.run(self.get_command(), shell=True)

    def align(self, alignment: str):
        self._add_to_flag_set(self.arguments.text.align, alignment)
        return self

    def vertical(self):
        self.__vertical = True
        return self



from gumpython.exceptions import BorderInputError

from .help_inputs.border_types import all_border_types
from .inputs import Input


class Border(Input):
    def __init__(self, border_type, flag_name):
        self.border_type = border_type
        self.flag_name = flag_name

    def is_valid(self):
        return self._validate()

    def _validate(self):
        if not isinstance(self.border_type, str):
            raise BorderInputError(
                f"'str' input expected for {self.flag_name} border type but '{type(self.border_type).__name__}' is given"
            )
        if self.border_type not in all_border_types():
            raise BorderInputError(
                f"border must be one of {all_border_types()} but got '{self.border_type}'"
            )
        return True

    def compile(self):
        return self.border_type



from gumpython.exceptions import FormatInputError

from .help_inputs.format_types import all_format_types
from .inputs import Input


class Format(Input):
    def __init__(self, format_type, flag_name):
        self.format_type = format_type
        self.flag_name = flag_name

    def is_valid(self):
        return self._validate()

    def _validate(self):
        if not isinstance(self.format_type, str):
            raise FormatInputError(
                f"'str' input expected for {self.flag_name} format type but '{type(self.format_type).__name__}' is given"
            )
        if self.format_type not in all_format_types():
            raise FormatInputError(
                f"border must be one of {all_format_types()} but got '{self.format_type}'"
            )
        return True

    def compile(self):
        return self.format_type



from typing import List, Union

from gumpython.arguments import StyleArgument
from gumpython.exceptions import StyleArgumentError
from gumpython.style import GumStyle

from .command import GumCommand


class Style(GumCommand):
    def __init__(
        self, text: Union[str, List[str]], gum_style_object: GumStyle
    ):
        super().__init__()
        self.argument = "style"
        self.text = text
        self._validate_text_argument()
        self.arguments = StyleArgument()
        self.gum_style_object = gum_style_object

    def _compile_command(self):
        self._compile_style(self.gum_style_object, self.arguments.args)
        super(Style, self)._compile_command()
        if isinstance(self.text, str):
            self.command.append(self.text)
        if isinstance(self.text, list):
            self.command.extend(self.text)

    def _validate_text_argument(self):
        if isinstance(self.text, str):
            return True
        if isinstance(self.text, list):
            for t in self.text:
                if not isinstance(t, str):
                    raise StyleArgumentError(f"{t} is not a string.")
            return True
        raise StyleArgumentError(
            f"Expected 'str' or List[str] but '{type(self.text).__name__}' is given."
        )

    def get_command(self):
        self._compile_style(self.gum_style_object, self.arguments.args)
        super(Style, self)._compile_command()
        if isinstance(self.text, str):
            self.command.append(f"'{self.text}'")
        if isinstance(self.text, list):
            self.command.extend(list(map(lambda x: f"'{x}'", self.text)))
        return " ".join(self.command)



from gumpython.constants import FlagTypeChoice

from .flags import BaseFlag, Flag


class InputCursorFlag(BaseFlag):
    def __init__(self):
        super(InputCursorFlag, self).__init__()

    def get_flag_name(self):
        return "cursor"


class InputPromptFlag(BaseFlag):
    def __init__(self):
        super(InputPromptFlag, self).__init__()
        self.prompt = Flag(name=self.flag_name, type=FlagTypeChoice.STR)

    def get_flag_name(self):
        return "prompt"


class InputGeneralFlag:
    def __init__(self):
        self.char_limit = Flag(name="char-limit", type=FlagTypeChoice.INT)
        self.password = Flag(name="password", type=FlagTypeChoice.BOOL)
        self.placeholder = Flag(name="placeholder", type=FlagTypeChoice.STR)
        self.value = Flag(name="value", type=FlagTypeChoice.STR)
        self.width = Flag(name="width", type=FlagTypeChoice.INT)



class GumPythonError(Exception):
    def __init__(self, message):
        super().__init__()
        self.message = message

    def __str__(self):
        return f"{self.message}"


class GumNotFoundError(GumPythonError):
    pass


class ArgumentError(GumPythonError):
    pass


class StyleArgumentError(ArgumentError):
    pass


class ChooseArgumentError(ArgumentError):
    pass


class ConfirmArgumentError(ArgumentError):
    pass


class FilterArgumentError(ArgumentError):
    pass


class FormatArgumentError(ArgumentError):
    pass


class JoinArgumentError(ArgumentError):
    pass


class SpinArgumentError(ArgumentError):
    pass


class InputError(GumPythonError):
    pass


class BorderInputError(InputError):
    pass


class ColorInputError(InputError):
    pass


class PositionInputError(InputError):
    pass


class AlignmentInputError(InputError):
    pass


class FormatInputError(InputError):
    pass


class SpinnerInputError(InputError):
    pass



from .arguments import (
    ChooseArgument,
    ConfirmArguments,
    FilterArguments,
    FormatArguments,
    InputArguments,
    JoinArguments,
    SpinArguments,
    StyleArgument,
    WriteArguments,
)



import subprocess

from .exceptions import GumNotFoundError, GumPythonError


def run_command(cmd, command_input: str = None):
    try:
        if command_input:
            response = subprocess.run(
                cmd, input=command_input, stdout=subprocess.PIPE, text=True
            )
        else:
            response = subprocess.run(cmd, stdout=subprocess.PIPE, text=True)
        return response
    except FileNotFoundError as fe:
        if "No such file or directory: 'gum'" in str(fe):
            raise GumNotFoundError(
                "No gum installation found. Please install gum cli. See README for dependencies."
            )
        raise GumPythonError(str(fe))
